#!/usr/bin/env python3

from System import *
from System.Diagnostics import *
from System.IO import *

import os

from Deadline.Plugins import *
from Deadline.Scripting import *

from pathlib import Path

def GetDeadlinePlugin():
    return HuskStandalone()

def CleanupDeadlinePlugin(deadlinePlugin):
    deadlinePlugin.Cleanup()

class HuskStandalone(DeadlinePlugin):
    # functions inside a class must be indented in python - DT
    def __init__( self ):
        self.InitializeProcessCallback += self.InitializeProcess
        self.RenderExecutableCallback += self.RenderExecutable # get the renderExecutable Location
        self.RenderArgumentCallback += self.RenderArgument # get the arguments to go after the EXE


    def Cleanup( self ):
        del self.InitializeProcessCallback
        del self.RenderExecutableCallback
        del self.RenderArgumentCallback

    def InitializeProcess( self ):
        self.SingleFramesOnly=True
        self.StdoutHandling=True
        self.PopupHandling=False

        self.AddStdoutHandlerCallback("USD ERROR(.*)").HandleCallback += self.HandleStdoutError # detect this error
        self.AddStdoutHandlerCallback( r"ALF_PROGRESS ([0-9]+(?=%))" ).HandleCallback += self.HandleStdoutProgress

    # get path to the executable
    def RenderExecutable(self):
        return self.GetConfigEntry( "USD_RenderExecutable" ).replace("<ns_Pipe>", "Houdini " + self.GetPluginInfoEntry("HouVersion"))

    # get the settings that go after the filename in the render command, 3Delight only has simple options.
    def RenderArgument( self ):
        if self.GetPluginInfoEntry("RenderDelegate") == "Redshift":
            os.environ["HOUDINI_PACKAGE_DIR"] = self.GetPluginInfoEntry("HouPackageLocation") # set delegate location

            # construct fileName
            #this will only support 1 frame per task

            usdFile = self.GetPluginInfoEntry("SceneFile")
            usdFile = RepositoryUtils.CheckPathMapping( usdFile )
            usdFile = usdFile.replace( "\\", "/" )

            usdPaddingLength = FrameUtils.GetPaddingSizeFromFilename( usdFile )

            frameNumber = self.GetStartFrame() # check this 2021 USD

            argument = ""
            
            argument += " --renderer HdRedshiftRendererPlugin"

            argument += " --usd-input " + usdFile

            argument += " --verbose a{}".format(self.GetPluginInfoEntry("LogLevel"))  # alfred style output and full verbosity

            argument += " --frame {}".format(frameNumber)

            argument += " --frame-count 1" # only render 1 frame per task

            ## renderer handled in job file. ##
            
            if self.GetPluginInfoEntry("OutImage") != "":
                if not os.path.isdir(os.path.dirname(self.GetPluginInfoEntry("OutImage"))):
                    os.makedirs(os.path.dirname(self.GetPluginInfoEntry("OutImage")))

                output_path = os.path.dirname(self.GetPluginInfoEntry("OutImage"))
                out_image_path_parts = self.GetPluginInfoEntry("OutImage").split("/")
                image_comp_parts = out_image_path_parts[-1].split(".")
                image_name = image_comp_parts[0]
                padded_frame_number = StringUtils.ToZeroPaddedString(frameNumber, len(image_comp_parts[-2]))
                image_format = image_comp_parts[-1]

                argument += " --output {0}/{1}.{2}.{3}".format(output_path, image_name, padded_frame_number, image_format)
            else:
                # fallback
                outputPath = os.path.dirname(usdFile).split('/') #[:-4] We are now going to site the composite USD in the project root.
                outputPath.append("render")
                outputPath = os.path.abspath(os.path.join(*outputPath))
            
                if not os.path.isdir(outputPath):
                    os.mkdir(outputPath)

                filename = Path(usdFile).name
                filename = Path(filename).with_suffix("")
                
                paddedFrameNumber = StringUtils.ToZeroPaddedString(frameNumber, 4)
                
                argument += " -o {0}/{1}.{2}.exr".format(outputPath, filename, paddedFrameNumber)

            argument += " --make-output-path"

            # argument += " --exrmode 0" ## legacy exr mode for fusion cryptomattes ##

            # argument += " --ocio 1"

            self.LogInfo( "Rendering USD file: " + usdFile )

        elif self.GetPluginInfoEntry("RenderDelegate") == "Octane":
            os.environ["HOUDINI_PACKAGE_DIR"] = self.GetPluginInfoEntry("HouPackageLocation") # set delegate location

            # construct fileName
            #this will only support 1 frame per task

            usdFile = self.GetPluginInfoEntry("SceneFile")
            usdFile = RepositoryUtils.CheckPathMapping( usdFile )
            usdFile = usdFile.replace( "\\", "/" )

            usdPaddingLength = FrameUtils.GetPaddingSizeFromFilename( usdFile )

            frameNumber = self.GetStartFrame() # check this 2021 USD

            argument = ""
            
            argument += " --renderer HdOctaneRendererPlugin"

            argument += " --usd-input " + usdFile

            argument += " --verbose a{}".format(self.GetPluginInfoEntry("LogLevel"))  # alfred style output and full verbosity

            argument += " --frame {}".format(frameNumber)

            argument += " --frame-count 1" # only render 1 frame per task

            ## renderer handled in job file. ##
            
            if self.GetPluginInfoEntry("OutImage") != "":
                if not os.path.isdir(os.path.dirname(self.GetPluginInfoEntry("OutImage"))):
                    os.makedirs(os.path.dirname(self.GetPluginInfoEntry("OutImage")))

                output_path = os.path.dirname(self.GetPluginInfoEntry("OutImage"))
                out_image_path_parts = self.GetPluginInfoEntry("OutImage").split("/")
                image_comp_parts = out_image_path_parts[-1].split(".")
                image_name = image_comp_parts[0]
                padded_frame_number = StringUtils.ToZeroPaddedString(frameNumber, len(image_comp_parts[-2]))
                image_format = image_comp_parts[-1]

                argument += " --output {0}/{1}.{2}.{3}".format(output_path, image_name, padded_frame_number, image_format)
            else:
                # fallback
                outputPath = os.path.dirname(usdFile).split('/') #[:-4] We are now going to site the composite USD in the project root.
                outputPath.append("render")
                outputPath = os.path.abspath(os.path.join(*outputPath))
            
                if not os.path.isdir(outputPath):
                    os.mkdir(outputPath)

                filename = Path(usdFile).name
                filename = Path(filename).with_suffix("")
                
                paddedFrameNumber = StringUtils.ToZeroPaddedString(frameNumber, 4)
                
                argument += " -o {0}/{1}.{2}.exr".format(outputPath, filename, paddedFrameNumber)

            argument += " --make-output-path"

            # argument += " --exrmode 0" ## legacy exr mode for fusion cryptomattes ##

            # argument += " --ocio 1"

            self.LogInfo( "Rendering USD file: " + usdFile )
        
        elif self.GetPluginInfoEntry("RenderDelegate") == "Karma":
            # construct fileName
            #this will only support 1 frame per task

            usdFile = self.GetPluginInfoEntry("SceneFile")
            usdFile = RepositoryUtils.CheckPathMapping( usdFile )
            usdFile = usdFile.replace( "\\", "/" )

            usdPaddingLength = FrameUtils.GetPaddingSizeFromFilename( usdFile )

            frameNumber = self.GetStartFrame() # check this 2021 USD

            argument = ""

            argument += " --renderer BRAY_HdKarma"

            argument += " --usd-input " + usdFile

            argument += " --verbose a{}".format(self.GetPluginInfoEntry("LogLevel"))  # alfred style output and full verbosity

            argument += " --frame {}".format(frameNumber)

            argument += " --frame-count 1" # only render 1 frame per task

            ## renderer handled in job file. ##
            
            if self.GetPluginInfoEntry("OutImage") != "":
                output_path = os.path.dirname(self.GetPluginInfoEntry("OutImage"))
                out_image_path_parts = self.GetPluginInfoEntry("OutImage").split("/")
                image_comp_parts = out_image_path_parts[-1].split(".")
                image_name = image_comp_parts[0]
                padded_frame_number = StringUtils.ToZeroPaddedString(frameNumber, len(image_comp_parts[-2]))
                image_format = image_comp_parts[-1]

                argument += " -o {0}/{1}.{2}.{3}".format(output_path, image_name, padded_frame_number, image_format)
            else:
                # fallback
                outputPath = os.path.dirname(usdFile).split('/') #[:-4] We are now going to site the composite USD in the project root.
                outputPath.append("render")
                outputPath = os.path.abspath(os.path.join(*outputPath))
            
                if not os.path.isdir(outputPath):
                    os.mkdir(outputPath)

                filename = Path(usdFile).name
                filename = Path(filename).with_suffix("")
                
                paddedFrameNumber = StringUtils.ToZeroPaddedString(frameNumber, 4)
                
                argument += " -o {0}/{1}.{2}.exr".format(outputPath, filename, paddedFrameNumber)

            argument += " --make-output-path"
            
            argument += " --exrmode 1" ## legacy exr mode for fusion cryptomattes ##

            # argument += " --ocio 1"

            self.LogInfo( "Rendering USD file: " + usdFile )

        elif self.GetPluginInfoEntry("RenderDelegate") == "KarmaXPU":
            # construct fileName
            #this will only support 1 frame per task

            usdFile = self.GetPluginInfoEntry("SceneFile")
            usdFile = RepositoryUtils.CheckPathMapping( usdFile )
            usdFile = usdFile.replace( "\\", "/" )

            usdPaddingLength = FrameUtils.GetPaddingSizeFromFilename( usdFile )

            frameNumber = self.GetStartFrame() # check this 2021 USD

            argument = ""

            argument += " --renderer BRAY_HdKarmaXPU"

            argument += " --usd-input " + usdFile

            argument += " --verbose a{}".format(self.GetPluginInfoEntry("LogLevel"))  # alfred style output and full verbosity

            argument += " --frame {}".format(frameNumber)

            argument += " --frame-count 1" # only render 1 frame per task

            ## renderer handled in job file. ##
            
            if self.GetPluginInfoEntry("OutImage") != "":
                output_path = os.path.dirname(self.GetPluginInfoEntry("OutImage"))
                out_image_path_parts = self.GetPluginInfoEntry("OutImage").split("/")
                image_comp_parts = out_image_path_parts[-1].split(".")
                image_name = image_comp_parts[0]
                padded_frame_number = StringUtils.ToZeroPaddedString(frameNumber, len(image_comp_parts[-2]))
                image_format = image_comp_parts[-1]

                argument += " -o {0}/{1}.{2}.{3}".format(output_path, image_name, padded_frame_number, image_format)
            else:
                # fallback
                outputPath = os.path.dirname(usdFile).split('/') #[:-4] We are now going to site the composite USD in the project root.
                outputPath.append("render")
                outputPath = os.path.abspath(os.path.join(*outputPath))
            
                if not os.path.isdir(outputPath):
                    os.mkdir(outputPath)

                filename = Path(usdFile).name
                filename = Path(filename).with_suffix("")
                
                paddedFrameNumber = StringUtils.ToZeroPaddedString(frameNumber, 4)
                
                argument += " -o {0}/{1}.{2}.exr".format(outputPath, filename, paddedFrameNumber)

            argument += " --make-output-path"
            
            argument += " --exrmode 1" ## legacy exr mode for fusion cryptomattes ##

            # argument += " --ocio 1"

            self.LogInfo( "Rendering USD file: " + usdFile )

        return argument

    # just incase we want to implement progress at some point
    def HandleStdoutProgress(self):
        self.SetStatusMessage(self.GetRegexMatch(0))
        self.SetProgress(float(self.GetRegexMatch(1)))

    # what to do when an error is detected.
    def HandleStdoutError(self):
        self.FailRender(self.GetRegexMatch(0))
